#include "header.h"

struct gfx *gfx;

#include <SDL.h>

#define HORIG 5
#define VORIG 5
#define HGRID 20
#define VGRID 2
#define HSIZE 15
#define VSIZE 1
#define MAXLI 50
#define MULTI (MAXLI/100.0)

SDL_Surface *screen;

int makecol(int r, int g, int b){
    SDL_PixelFormat *format;

    format=screen->format;
    return (((r>>format->Rloss)<<format->Rshift)&format->Rmask) |
           (((g>>format->Gloss)<<format->Gshift)&format->Gmask) |
           (((b>>format->Bloss)<<format->Bshift)&format->Bmask);
}

//#define CLIPPED(clip, _x, _y) ((_x)<(clip)->x || (_y)<(clip)->y || (_x)>=(clip)->x+(clip)->w || (_y)>=(clip)->y+(clip)->h) 
#define CLIPPED(clip, _x, _y) (0)

void fast_putpixel8(SDL_Rect *clip, int x, int y, int color){
    Uint8 *p;

    if CLIPPED(clip, x, y) return;
    p = (Uint8 *) screen->pixels + y*screen->pitch + x;
    *p = color;
}

void fast_putpixel16(SDL_Rect *clip, int x, int y, int color){
    Uint16 *p;

    if CLIPPED(clip, x, y) return;
    p = (Uint16 *) screen->pixels + y*screen->pitch/2 + x;
    *p = color;
}

void fast_putpixel24(SDL_Rect *clip, int x, int y, int color){
    Uint16 *p16;
    Uint8  *p8;

    if CLIPPED(clip, x, y) return;
    p16 = (Uint16 *) screen->pixels + y*screen->pitch/3 + x;
    p8  = (Uint8 *)  (p16+1); 
    *p16 = color;     /* TODO Big endian? */
    *p8  = color>>16; 
}

void fast_putpixel32(SDL_Rect *clip, int x, int y, int color){
    Uint32 *p;

    if CLIPPED(clip, x, y) return;
    p = (Uint32 *) screen->pixels + y*screen->pitch/4 + x;
    *p = color;
}

void (*fast_putpixel)(SDL_Rect *clip, int x, int y, int color);


int fast_getpixel8(int x, int y){
    Uint8 *p;

    p = (Uint8 *) screen->pixels + y*screen->pitch + x;
    return *p;
}

int fast_getpixel16(int x, int y){
    Uint16 *p;

    p = (Uint16 *) screen->pixels + y*screen->pitch/2 + x;
    return *p;
}

int fast_getpixel24(int x, int y){
    Uint16 *p16;
    Uint8  *p8;

    p16 = (Uint16 *) screen->pixels + y*screen->pitch/3 + x;
    p8  = (Uint8 *)  (p16+1); 
	return (*p16) + ((*p8)<<16);
//    *p16 = color;     /* TODO Big endian? */
//    *p8  = color>>16; 
}

int fast_getpixel32(int x, int y){
    Uint32 *p;

    p = (Uint32 *) screen->pixels + y*screen->pitch/4 + x;
    return *p;
}

int (*fast_getpixel)(int x, int y);



void line(SDL_Rect *clip, int x1, int y1, int x2, int y2, int color){
    int dx, dy, p;	
    int inc, tmp;

   // dbg("line(%d,%d,%d,%d,%6x)\n", x1,y1,x2,y2,color);
    
    dx=abs(x1-x2);
    dy=abs(y1-y2);
    if (dx>=dy){
    	p = 2*dy-dx;
    	if (x1 >= x2){   
            tmp=x1;x1=x2;x2=tmp;
	    	tmp=y1;y1=y2;y2=tmp;
		}
        fast_putpixel(clip, x1, y1, color);
    	if (y2>=y1) inc=1;
        else inc=-1;
    	
        while(x1 < x2){	
    	    x1++;   	
            if (p<0) 
                p += 2*dy;
            else{
                y1+=inc;
                p += 2*(dy-dx);
            }
            fast_putpixel(clip, x1, y1, color);
        }
    }else{
    	p = 2*dx-dy;
        if (y1 >= y2){
            tmp=x1;x1=x2;x2=tmp;
	    	tmp=y1;y1=y2;y2=tmp;
        }
        fast_putpixel(clip, x1, y1, color);
    	
    	if (x2>=x1) inc=1;
        else inc=-1;
    	
        while(y1 < y2){	
    	    y1++;   	
            if (p<0) 
                p += 2*dx;
            else{
                x1 += inc;
                p += 2*(dx-dy);
            }
            fast_putpixel(clip, x1,y1, color);
        }
    }
} 

void do_line(SDL_Rect *clip, int x1, int y1, int x2, int y2, int color, 
        void (*func)(SDL_Rect *clip, int x, int y, int d) ){

    int dx, dy, p;	
    int inc, tmp;
    
    dx=abs(x1-x2);
    dy=abs(y1-y2);
    if (dx>=dy){
    	p = 2*dy-dx;
    	if (x1 >= x2){   
            tmp=x1;x1=x2;x2=tmp;
	    	tmp=y1;y1=y2;y2=tmp;
		}
        func(clip, x1, y1, color);
    	if (y2>=y1) inc=1;
        else inc=-1;
    	
        while(x1 < x2){	
    	    x1++;   	
            if (p<0) 
                p += 2*dy;
            else{
                y1+=inc;
                p += 2*(dy-dx);
            }
            func(clip, x1, y1, color);
        }
    }else{
    	p = 2*dx-dy;
        if (y1 >= y2){
            tmp=x1;x1=x2;x2=tmp;
	    	tmp=y1;y1=y2;y2=tmp;
        }
        func(clip, x1, y1, color);
    	
    	if (x2>=x1) inc=1;
        else inc=-1;
    	
        while(y1 < y2){	
    	    y1++;   	
            if (p<0) 
                p += 2*dx;
            else{
                x1 += inc;
                p += 2*(dx-dy);
            }
            func(clip, x1,y1, color);
        }
    }
} 

/********************************************/

int SetVideoMode(int w, int h){
	SDL_Color colors[256];

    if ((screen = SDL_SetVideoMode(w, h, 0, SDL_SWSURFACE|SDL_HWPALETTE))==NULL){
        return 1;    
    }

    if (screen->format->palette){
        int i;
        SDL_PixelFormat *fmt;
        
        for (i=0; i<256; i++){
            colors[i].r=(i&0xe0)+31; 
            colors[i].g=((i<<3)&0xe0)+31;
            colors[i].b=((i<<6)&0xc0)+63;
        }
        SDL_SetColors(screen, colors, 0, 256);

        fmt=screen->format;
        fmt->Rmask=0xe0;
        fmt->Gmask=0x1c;
        fmt->Bmask=0x03;
        fmt->Rshift=5;
        fmt->Gshift=2;
        fmt->Bshift=0;
        fmt->Rloss=5;
        fmt->Gloss=5;
        fmt->Bloss=6;
        
        
    }
    
    SDL_WM_SetCaption("ssbm1", "ssbm2");
    return 0;
}

int init_gfx(){
    int w,h;

    
    SDL_Init(SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE); 

    w=2*VORIG+MAX_CHANNELS*HGRID;
    h=2*HORIG+MAXLI*VGRID;
    
    if (SetVideoMode(w, h)){
        dbg("Can't set graphics mode %dx%d\n", w,h);
        return -1;
    }
    
    

    switch(screen->format->BytesPerPixel){
        case 1:
            fast_putpixel = fast_putpixel8;
			fast_getpixel = fast_getpixel8;
            break;
        case 2:
            fast_putpixel = fast_putpixel16;
			fast_getpixel = fast_getpixel16;
            break;
        case 3:
            fast_putpixel = fast_putpixel24;
			fast_getpixel = fast_getpixel24;
            break;
        case 4:
            fast_putpixel = fast_putpixel32;
			fast_getpixel = fast_getpixel32;
            break;
    }

    SDL_UpdateRect(screen, 0, 0, 0, 0);
    
//SDL_SetEventFilter(FilterEvents);
    SDL_EventState(SDL_KEYUP, SDL_IGNORE);
    SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
    return 0;
}
                                                                             



void free_gfx(){
    SDL_FreeSurface(screen);
    SDL_Quit();
}


int gfx_redraw(){
	int i,j,c;

	for (i=0;i<channels;i++){
		for (j=0;j<=MAXLI;j++){
			if (((double)volumes[i]*MULTI)>j){
				if (j>80*MULTI)
					c=makecol(200,0,0);
				else
					c=makecol(0,200,0);
				
			}else
				c=makecol(100,100,100);
			
	  		line(NULL, 
				HORIG+i*HGRID,	
				VORIG+(MAXLI-j)*VGRID,
				HORIG+i*HGRID+HSIZE,
				VORIG+(MAXLI-j)*VGRID,
				c);
		}
	}
	
    SDL_UpdateRect(screen, 0, 0, 0, 0);
    return 0;
}

int gfx_func(){
    SDL_Event event,*ev;
    int konec=0;
    
    
    ev=&event;
    
    while (SDL_PollEvent(ev)) {
    
    //  dbg("event: %d\n", ev->type);
        switch (ev->type){
            case SDL_KEYDOWN:
                switch(ev->key.keysym.sym){
                    case SDLK_q:
                        konec=1;
                        break;
                    default: /* for warning */
                        break;
                }
                break /* SDL_KEYDOWN */;
            case SDL_VIDEORESIZE:
                SetVideoMode(ev->resize.w, ev->resize.h);
                break;
			case SDL_QUIT:
                konec=1;
				break;
        }

        if (konec) {
            free_gfx();
            return -1;
        }
    }
    return 0;
}


