#include "header.h"

#include <grp.h>
#include <string.h>
struct mixer *mix;

int udpsock;
int udpport=NET_PORT;
int volumes[MAX_CHANNELS];
int channels;
char *host;
struct sockaddr_in hostsin;

int init_net_udp(void){
    int on;
    //socklen_t socklen;
    struct hostent *he;

    udpsock=-1;
    
    host=strdup("localhost");
    he=gethostbyname(host);
    if (!he){
        error("Can't resolve %s",host);
        goto err;
    }

    hostsin.sin_family=AF_INET;
    memcpy(&hostsin.sin_addr, he->h_addr_list[0], he->h_length);;
    hostsin.sin_port=htons(udpport);

    udpsock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (udpsock < 0) goto err;

    on=1;
/*    if (setsockopt(udpsock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))){
        error("Can't set SO_REUSEADDR");
        goto err;
    }*/
    
    if (fcntl(udpsock, F_SETFL, O_NONBLOCK)){
        error("Can't set O_NONBLOCK");
        goto err;
    }
    
    set_handlers(udpsock, udp_read_handler, NULL, udp_exception_handler, NULL);

    return 0;
err:;
    if (udpsock>=0) close(udpsock);
    udpsock = -1;
    
    /* kill_timer not needed */
    return 1;    
}


void free_udp(){
    set_handlers(udpsock, NULL, NULL, NULL, NULL);
    if (udpsock>=0) close(udpsock);
    udpsock=-1;
}

#define DELIM " \r\n"


void udp_read_handler(void *data){
    char s[1024],*token_ptr,*c;
    struct sockaddr_in sin;
    socklen_t socklen;
    int rcvd,i;
    char *id,*chans;
    
    
   // dbg("udp_read_handler()\n");

    memset(s, 0, sizeof(s));
    socklen = sizeof(sin);
    rcvd=recvfrom(udpsock, s, sizeof(s)-1, 0, 
            (struct sockaddr *)&sin, &socklen);

    if (rcvd<0) return;
    
    if (strlen(s)>0 && s[strlen(s)-1]=='\n') s[strlen(s)-1]='\0';
    if (strlen(s)>0 && s[strlen(s)-1]=='\r') s[strlen(s)-1]='\0';
    if (*s=='\0') return;
    dbg("read: '%s'\n", s);

  //  strcpy(s, "0 2 90 50");
    
    id=strtok_r(s, DELIM, &token_ptr);
    if (!id) return;
    chans=strtok_r(NULL, DELIM, &token_ptr);
    if (!chans) return;
    channels=atoi(chans);
    if (channels<0) channels=0;
    if (channels>MAX_CHANNELS) channels=MAX_CHANNELS;
    
    for (i=0;i<channels;i++){
        c=strtok_r(NULL, DELIM, &token_ptr);
        if (!c) break;
        volumes[i]=atoi(c);
    }
    gfx_redraw();
            
}    

void udp_exception_handler(void *data){
    dbg("udp_exception_handler()\n");
}

void sigalrm(int sig){
    dbg("SIGALRM!\n");
}

void sigint(int sig){
    dbg("SIGINT!\n");
    //play_abort();
    exit(1);
}

void sigterm(int sig){
    dbg("SIGTERM!\n");
    //play_abort();
    exit(1);
}

void init(){
    debug_type=0;
    init_debug();
    set_sigcld();
    signal(SIGALRM, sigalrm);
    signal(SIGINT,  sigint);
    signal(SIGTERM, sigterm);
    if (init_net_udp()) {
        terminate_all_subsystems();
        exit(1);
    }
    if (init_gfx()){
        terminate_all_subsystems();
        exit(1);
    }
    install_timer_(200, gfx_timer, NULL);
    
}

void terminate_all_subsystems(void){
    free_gfx();
    check_bottom_halves();
    check_bottom_halves();
    free_udp();
}

char *parse_options(int argc, char **argv){
    int c;
    char *s;

    s=getenv("SSBC_DEBUG");
    if (s){
        if (strlen(s)!=0){
            debug_type=1;
            debug_filename=s;
        }else{
            debug_type=2;
        }
    }
    
    while (1){
            
        int option_index = 0;
        static struct option long_options[] = {
            {"debug", 2, 0, 'd'},
            {"port", 1, 0, 'p'},
            {"version", 0, 0, 'v'},
            {0, 0, 0, 0}
        };

        c = getopt_long (argc, argv, "d:hp:v?", long_options, &option_index);
        if (c == -1) break;
     /*   printf("c='%c\n",c);
       printf("optarg=%p '%s'\n", optarg,optarg);*/

        switch (c){
            case 0:
                printf ("option %s", long_options[option_index].name);
                if (optarg) printf (" with arg %s", optarg);
                printf ("\n");
                break;
            
            case 'd':
                if (optarg) {
                    debug_type=1;
                    debug_filename=optarg;
                }else{
                    debug_type=2;
                }
                break;

            case 'h':     
            case '?':
                printf("Options: ssbd -[dh?knpv]\n");
                
                exit(0);
                break;
           
            case 'p':
                udpport=atoi(optarg);
                break;
                
            case 'v':
                printf ("%s version %s\n", PACKAGE, VERSION);
                printf ("http://tucnak.nagano.cz/ssbd.html\n");
                
                exit(0);
                break;


            default:
                printf ("?? getopt returned character code 0%o ??\n", c);
        }
    }

    if (optind < argc){
        printf ("non-option ARGV-elements: ");
        while (optind < argc) printf ("%s ", argv[optind++]);
        printf ("\n");
    }

    return NULL;
}


void gfx_timer(void *arg){
    char buf[256];
    
    if (gfx_func()){
        terminate_all_subsystems();
        exit(0);
        return;
    }

    strcpy(buf, "0 1 ");
    sendto(udpsock, buf, strlen(buf), 0, (struct sockaddr *)&hostsin, sizeof(hostsin)); 
                        
    install_timer_(200, gfx_timer, NULL);
}


int main(int argc, char **argv){

    parse_options(argc, argv);
    
    if (udpport<=0 || udpport>65535){
        printf("UDP port must be 1..65535\n");
        exit(1);
    }

    select_loop(init);
    terminate_all_subsystems();
    return 0;
}    
    

